/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 *
 */
package org.xwiki.eclipse.storage.utils;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.xwiki.eclipse.core.CoreLog;
import org.xwiki.eclipse.storage.StoragePlugin;

import com.google.gson.reflect.TypeToken;

/**
 * A class that implements a Map from strings to strings, that saves its content to the disk.
 * 
 * @version $Id: 1549d187707eb854ae4bd8fa91b83cc30b48c975 $
 */
public class PersistentMap
{
    private IFile file;

    private Map<String, String> map;

    /**
     * Constructor.
     * 
     * @param file The file where to synchronize the map content.
     * @throws CoreException
     */
    public PersistentMap(IFile file) throws CoreException
    {
        this.file = file;

        if (file.exists()) {
            try {
                map = (Map<String, String>) StorageUtils.readFromJson(file, new TypeToken<Map<String, String>>()
                {
                }.getType());
            } catch (Exception e) {
                throw new CoreException(new Status(IStatus.ERROR, StoragePlugin.PLUGIN_ID,
                    "Unable to initialize persistent map", e));
            }
        } else {
            map = new HashMap<String, String>();
            synchronize();
        }
    }

    public void put(String key, String value) throws CoreException
    {
        map.put(key, value);
        synchronize();
    }

    public void remove(String key) throws CoreException
    {
        if (!map.containsKey(key)) {
            return;
        }

        map.remove(key);
        synchronize();
    }

    public String get(String key)
    {
        return map.get(key);
    }

    public Set<String> keySet()
    {
        return map.keySet();
    }

    private synchronized void synchronize() throws CoreException
    {       
            StorageUtils.writeToJson(file, map);      
    }
}
